#! /usr/bin/perl
#
# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use warnings FATAL => 'all';
use diagnostics;

require "html.pl";
require "http.pl";
require "db.pl";
require "dbops.pl";
require "url.pl";
require "common.pl";
require "lang.pl";
require "urllist.pl";
require "session.pl";
require "op_chan.pl";
require "op_nick.pl";

my $max_days = 90;
my $top_n = 10;


&main();
exit(0);


sub main()
{
	&HTTP::print_header();
	&HTML::print_header();

	my ($chan_id, $chan_name) = &Session::guess_channel();
	&Session::save_channel(&Session::get_session(), $chan_id);
	&Session::print_links(undef, &URL::stat($chan_id));

	printf "<h1>%s</h1>\n", &Lang::get("stat-title");

	my $url = &HTTP::get("str", "");	# (part of) URL to search

	my $url_pos = &HTTP::get("pos", "");	# bit: 1: begin, 2: end
	$url_pos = 0 if (! ($url_pos =~ /./));

#	my $url_strip = &HTTP::get("url_s", 1);	# strip www.

	my $age_max = &HTTP::get_numeric("max_a", 90);
	$age_max = 1 if ($age_max < 1);

	my $age_min = &HTTP::get_numeric("min_a", 0);

	# 0: urls per user (user top n)
	# 1: urls per hour per user (hour 0-23)
	# 2: domains per user (domain top n)
	my $stat = &HTTP::get_numeric("r", 0);

	if ($age_max - $age_min > $max_days) {
		$age_min = $age_max - $max_days;
	}

	my $r;
	if ($url =~ /./ && defined($chan_id)) {
		if ($stat == 0) {
			$r = &user_top($chan_id, $url, $url_pos,
					$age_max, $age_min);
		}
	}

	&query_page($chan_name, $url, $age_max, $age_min);

	&HTML::print_footer(&DB::process_time());
}


# 0: chan
# 1: url
# 2: age_max
# 3: age_min
sub query_page($$$$)
{
	my ($d_chan, $d_url, $d_age_max, $d_age_min) = @_;

	my $l_chan = &Lang::get("col-channel");
	my $l_url = &Lang::get("col-url");
	my $l_age = &Lang::get("col-age");
	my $l_max = &Lang::get("txt-age-max");
	my $l_min = &Lang::get("txt-age-min");
	my $l_days = &Lang::get("txt-days");
	my $l_search = &Lang::get("txt-search");

	if (! defined($d_chan) || ! ($d_chan =~ /./)) {
		$d_chan = &Lang::get("txt-any");
	}
	$d_url = "" if (! defined($d_url));
	$d_age_max = $max_days if (! defined($d_age_max));
	$d_age_min = 0 if (! defined($d_age_min));

	print "<h2>" . &Lang::get("stat-search") . "</h2>\n\n";

	my $self = &URL::stat();
	$self =~ s/.*\///;
	print <<EOF;
<form action="$self" method="get">
<table class="search">
	<tr>
		<th>$l_chan</th>
		<td colspan="2"><input type="text" name="chan" size="40"
				value="$d_chan" /></td>
	</tr>
	<tr>
		<th>$l_url</th>
		<td colspan="2"><input type="text" name="str" size="40"
				value="$d_url" /></td>
	</tr>
	<tr>
		<th rowspan="2">$l_age</th>
		<td>$l_max</td>
		<td class="max"><input type="text" name="max_a" size="4"
				value="$d_age_max" />$l_days</td>
	</tr>
	<tr>
		<td>$l_min</td>
		<td class="max"><input type="text" name="min_a" size="4"
					value="$d_age_min" />$l_days</td>
	</tr>
	<tr>
		<th></th>
		<td><input type="submit" value="$l_search" /></td>
	</tr>
</table>
</form>
EOF
}


# 0: chan
# 1: url
# 2: url_pos
# 3: age_max
# 4: age_min
sub user_top($$$$$)
{
	my ($chan, $url_str, $url_pos, $age_max, $age_min) = @_;
	my ($query, $result);

	printf "<p>" . &Lang::get("stat-top-n-users") . " ", $top_n;

	if ($url_pos == 3) {
		printf &Lang::get("stat-urls-exactly-like"), $url_str;
	} else {
		my $url_match;
		if ($url_pos == 0) {
			$url_match = "*$url_str*";
		} elsif ($url_pos == 1) {
			$url_match = "$url_str*";
		} elsif ($url_pos == 2) {
			$url_match = "*$url_str";
		}
		printf &Lang::get("stat-urls-like"), $url_match;
	}

	print " ";
	if ($chan == -1) {
		printf &Lang::get("stat-on-any-channel");
	} else {
		my $chan_name = &Chan::get_name($chan);
		printf &Lang::get("stat-on-channel"), $chan_name;
	}

	print " ";
	printf &Lang::get("stat-aged-max"), $age_max;
	if ($age_min != 0) {
		printf " " . &Lang::get("stat-aged-min"), $age_min;
	}
	print &Lang::get("stat-fullstop") . "</p>\n\n";


	$query = "SELECT id, nick FROM urls WHERE ";
	$query .= "chan = $chan AND " if ($chan != -1);

	$query .= "CURRENT_TIMESTAMP - time < INTERVAL '$age_max days' AND ";
	if ($age_min > 0) {
		$query .= "CURRENT_TIMESTAMP - time >= " .
				"INTERVAL '$age_min days' AND ";
	}

	$query .= "url ILIKE '\%$url_str\%'";

	$result = &DB::query($query);
	if ($result->ntuples == 0) {
		print "<p>" . &Lang::get("stat-no-matches") . "</p>\n\n";
		return 0;
	}

	my $url_count = 0;
	my %user_url;
	while (my ($url, $user) = $result->fetchrow) {
		($url_pos == 1 && ! ($url =~ /^$url_str/i)) && next;
		($url_pos == 2 && ! ($url =~ /$url_str$/i)) && next;
		($url_pos == 3 && ! ($url =~ /^$url_str$/i)) && next;

		$user_url{$user}++;
		$url_count++;
	}

	my (@top_user, @top_count);
	while (my ($user, $count) = each %user_url) {
		push(@top_user, $user);
		push(@top_count, $count);
	}

	for (my $i = 0; $i <= $#top_user - 1; $i++) {
		my $ti = $i;
		my $tc = $top_count[$i];
		for (my $j = $i + 1; $j <= $#top_user; $j++) {
			if ($top_count[$j] > $tc) {
				$tc = $top_count[$j];
				$ti = $j;
			}
		}
		if ($ti != $i) {
			my $t;
			$t = $top_user[$i];
			$top_user[$i] = $top_user[$ti];
			$top_user[$ti] = $t;
			$t = $top_count[$i];
			$top_count[$i] = $top_count[$ti];
			$top_count[$ti] = $t;
		}
	}

	my %users = &Nick::get_names(@top_user);

	my $l_nick = &Lang::get("col-nick");
	my $l_count = &Lang::get("col-times-pasted");
	my $l_of_all = &Lang::get("col-share-of-all");
	print <<EOF;
<table class="pasters">
	<tr>
		<th>$l_nick</th>
		<th>$l_count</th>
		<th>$l_of_all</th>
	</tr>
EOF
	my $d_sum = 0;
	for (my $i = 0; $i < $top_n && $i <= $#top_user; $i++) {
		my $p_user = $users{$top_user[$i]};
		my $l_user = &URL::user($top_user[$i]);
		my $p_count = $top_count[$i];
		my $p_perc = sprintf("%.1f", $p_count / $url_count * 100);
		my $class = $i % 2 == 1 ? " class=\"zebra\"" : "";
		$d_sum += $p_count;
		print <<EOF;
	<tr$class>
		<td><a href="$l_user">$p_user</a></td>
		<td>$p_count</td>
		<td>$p_perc%</td>
	</tr>
EOF
	}
	my $l_total = &Lang::get("col-total");
	my $d_perc = sprintf("%.1f", $d_sum / $url_count * 100);
	print <<EOF;
	<tr class="sum">
		<td>$l_total</th>
		<td>$d_sum</td>
		<td>$d_perc%</td>
	</tr>
</table>
EOF

	return 1;
}
