#! /usr/bin/perl
#
# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use warnings FATAL => 'all';
use diagnostics;

require "html.pl";
require "http.pl";
require "db.pl";
require "dbops.pl";
require "url.pl";
require "common.pl";
require "lang.pl";
require "urllist.pl";
require "session.pl";
require "op_nick.pl";



&main();
exit(0);


sub main
{
	&HTTP::print_header();
	&HTML::print_header();

	my $user = &HTTP::get_numeric("user", -1);
	my $seek = &HTTP::get_numeric("seek", -1);
	my $n = &HTTP::get_numeric("n", -1);

	&Session::print_links(undef, &URL::user($user, $n, $seek));

	if ($seek == -1) {
		&summary($user);
	} else {
		&url_hist($user, $n, $seek);
	}

	&HTML::print_footer(&DB::process_time());
}


# 0: user
sub summary
{
	my ($id) = @_;

	my ($nick) = &Nick::get_name($id);
	(defined($nick)) || &HTML::bad_param("bad user");

	printf "<h1>" . &Lang::get("user-summary") . "</h1>\n", $nick;

	my @score = &get_score($id, 10);
	($#score != -1) && &print_score(@score);

	my $r = &urls($id, 0, 8);
	if ($r > 0) {
		&urls($id, 1, 4);
	} else {
		printf "<p>" . &Lang::get("user-no-urls") . "</p>\n", $nick;
	}
}


# 0: user
# 1: n
# 2: seek
sub url_hist
{
	my ($id, $n, $seek) = @_;

	my $nick = &Nick::get_name($id);
	(defined($nick)) || &HTML::bad_param("bad user");

	printf "<h1>" . &Lang::get("user-hist-for") . "</h1>\n", $nick;

	&urls($id, 0, $n, $seek);
}


# 0: array of hash
sub print_score
{
	my @chans = @_;

	my $l_chan = &Lang::get("col-channel");
	my $l_fame = &Lang::get("col-fame");
	my $l_shame = &Lang::get("col-shame");
	my $l_urls = &Lang::get("col-urls");
	my $l_ratio = &Lang::get("col-points-ratio");
	my $l_wanha = &Lang::get("col-wanha");
	my $l_cheat = &Lang::get("col-cheat");

	print <<EOF;
<table class="perchan">
	<tr class="ul">
		<th>$l_chan</th>
		<th>$l_urls</th>
		<th>$l_fame</th>
		<th>$l_ratio</th>
		<th>$l_shame</th>
		<th>$l_ratio</th>
		<th>$l_wanha</th>
		<th>$l_cheat</th>
	</tr>
EOF

	my $t_fame = 0;
	my $t_shame = 0;
	my $t_wanha = 0;
	my $t_cheat = 0;
	my $t_urls = 0;
	for (my $i = 0; $i <= $#chans; $i++) {
		my %t = %{$chans[$i]};

		my $id = $t{'id'};
		my $name = $t{'name'};
		my $fame = $t{'fame'};
		my $shame = $t{'shame'};
		my $wanha = $t{'wanha'};
		my $cheat = $t{'cheat'};
		my $count = $t{'url_count'};

		my $a_chan = &URL::chan($id);
		my $class = $i % 2 == 0 ? "zebra_c" : "center";

		my $total = $count + $shame + $wanha;
		my $f_ratio = sprintf("%.1f", $fame / $total * 100);
		my $s_ratio = sprintf("%.1f", $shame / $total * 100);

		print <<EOF;
	<tr class="$class">
		<td class="left"><a href="$a_chan">$name</a></td>
		<td>$total</td>
		<td>$fame</td>
		<td>$f_ratio%</td>
		<td>$shame</td>
		<td>$s_ratio%</td>
		<td>$wanha</td>
		<td>$cheat</td>
	</tr>
EOF
		$t_fame += $fame;
		$t_shame += $shame;
		$t_urls += $total;
		$t_wanha += $wanha;
		$t_cheat += $cheat;
		$i++;
	}

	my $l_total = &Lang::get("col-total");
	print <<EOF;
	<tr class="sum">
		<td>$l_total</td>
		<td class="center">$t_urls</td>
		<td class="center">$t_fame</td>
		<td class="center"></td>
		<td class="center">$t_shame</td>
		<td class="center"></td>
		<td class="center">$t_wanha</td>
		<td class="center">$t_cheat</td>
	</tr>
</table>
EOF
}


# 0: user
# 1: mode, 0 == latest, 1 == top-n
# 2: n
# 3: seek
# r: 0 if urls, 1 if no urls
sub urls
{
	my ($user_id, $mode, $n, $seek) = @_;
	my @urls;
	my @images;
	my $has_more;

	$n = 25 if ($n > 25);

	# get one extra so that we know if there's more
	@urls = &get_urls($user_id, $mode, $n + 1, $seek);
	($#urls == -1 && defined($seek)) && &HTML::bad_param("bad seek");
	($#urls == -1) && return 0;
	if ($#urls == $n) {
		$has_more = 1;
		pop @urls;
	} else {
		$has_more = 0;
	}

	if (! defined($seek)) {
		my $fmt = &Lang::get(($mode == 0) ?
				"user-n-latest-urls" : "user-n-best-urls");
		printf "<h2>" . $fmt . "</h2>\n", $n;
	} else {
		if ($mode == 0) {
			my ($first) = $urls[0] =~ /^(\d+),/;
			my ($last) = $urls[$#urls] =~ /^(\d+),/;
			printf "<h2>" . &Lang::get("user-urls-range") .
					"</h2>\n", $first, $last;
		} else {
			my $first = $seek + 1;
			my $last = $n + $seek;
			printf "<h2>" . &Lang::get("user-pop-urls-range") .
					"</h2>\n", $first, $last;
		}
	}

	@images = &URLList::print(\@urls, "url", "chan", "date", "count");

	($mode == 1) && return 1;

	my @links;
	my $l;
	if (! defined($seek) && $has_more == 1) {
		push(@links, &URL::user($user_id, 25, $n),
				&Lang::get("link-older-urls"));
	} elsif (defined($seek)) {
		push(@links, &URL::user($user_id),
				&Lang::get("link-user-summary"));
		if ($seek > 0) {
			my $tseek = $seek - $n;
			$tseek = 0 if ($tseek < 0);
			push(@links, &URL::user($user_id, $n, $tseek),
					&Lang::get("link-newer"));
		}
		if ($has_more == 1) {
			my $tseek = $seek + $n;
			push(@links, &URL::user($user_id, $n, $tseek),
					&Lang::get("link-older"));
		}
	}

	&Common::print_links(@links);
	&Common::link_images(@images);

	return 1;
}


# 0: user
# 1: mode, 0 == latest, 1 == top-n
# 2: n
# 3: seek
# r: array of hash (url_id, url_url, url_date, url_count, chan_id, chan_name)
sub get_urls
{
	my ($user_id, $mode, $n, $seek) = @_;
	my ($query, $result);
	my @ret;

	my $order;
	if ($mode == 0) {
		$order = "urls.time";
	} elsif ($mode == 1) {
		$order = "urls.count";
	} else {
		&HTML::bad_param("mode");
	}

	$query = "SELECT urls.url, urls.id, " .
				&DBOps::get_date_fmt("urls.time") . ", " .
				"urls.count, chans.id, chans.chan " .
			"FROM urls, chans " .
			"WHERE urls.nick = $user_id " .
				"AND urls.chan = chans.id " .
			"ORDER BY $order DESC " .
			"LIMIT $n";
	$query .= " OFFSET $seek" if (defined($seek));

	$result = &DB::query($query);
	($result->ntuples == 0) && return ();

	while (my ($url_url, $url_id, $url_date, $url_count,
			$chan_id, $chan_name) = $result->fetchrow) {
		my %t = (
			'url_id' => $url_id,
			'url_url' => $url_url,
			'url_date' => $url_date,
			'url_count' => $url_count,
			'chan_id' => $chan_id,
			'chan_name' => $chan_name
			);
		push(@ret, \%t);
	}
	return @ret;
}


# 0: nick id
# 1: n
# r: array of hash (id, url_count, fame, shame, wanha, cheat, name)
sub get_score
{
	my ($id, $n) = @_;

	my $query = "SELECT chans.id, chans.chan, " .
				"sb.fame, sb.shame, sb.wanha, sb.cheat " .
			"FROM chans, scoreboard AS sb " .
			"WHERE sb.nick = $id AND sb.chan = chans.id " .
			"ORDER BY sb.fame, sb.shame DESC ".
			"LIMIT $n";
	my $result = &DB::query($query);
	if ($result->ntuples == 0) {
		return;
	}

	my @ret;
	while (my ($id, $name, $fame, $shame, $wanha, $cheat) =
			$result->fetchrow) {
		my %t = (
			'id' => $id,
			'name' => $name,
			'fame' => $fame,
			'shame' => $shame,
			'cheat' => $cheat,
			'wanha' => $wanha
			);
		push(@ret, \%t);
	}

	foreach (@ret) {
		my %t = %{$_};
		my $chan = $t{'id'};
		$query = "SELECT COUNT(id) FROM urls " .
				"WHERE nick = $id AND chan = $chan";
		$result = &DB::query($query);
		my ($c) = $result->fetchrow;
		# TODO there must be a prettier way to do this
		${%{$_}}{'url_count'} = $c;
	}

	return @ret;
}
